#include "doorkeeper.hpp"
#include "doorkeeperConfig.h"
#include "platform.h"
#include "platformstrings.h"
#include <iostream>
#include <fstream>
#include <memory>

int main() {
	typedef dk::Tyler<2> Tiler;
	typedef std::unique_ptr<std::ifstream> ifstreamptr;

	std::cout << "Welcome to " << APP_NAME << ' ' << DK_DEVICE_STRING << " version for " << DK_OS_STRING << ' ' << DK_ARCH_STRING << ' ' << DK_BIT_STRING;
#if defined(DK_ARM)
	std::cout << ' ' << DK_ARM_FAMILY_STRING << ' ' << DK_ARM_ARCH_STRING;
#endif
	std::cout << '\n';

	Tiler tiler(Tiler::coords(10, 6), Tiler::coords(64));

	const std::string map1(DATA_PATH"/test.map");
	const std::string map2(DATA_PATH"/test_2.map");

	std::cout << "Loading " << map1 << '\n';
	auto bottom_layer = &tiler.push_layer<int>(dk::Layer<int, 2>::streamer_type(ifstreamptr(new std::ifstream(map1))));
	//std::cout << "Loading " << map2 << '\n';
	//tiler.push_layer<char>(dk::Layer<char, 2>::streamer_type(ifstreamptr(new std::ifstream(map2))), dk::Vector<int, 2>(2));

	dk::Viewport<2> viewport(tiler, Tiler::coords(0), Tiler::coords(10, 6));
	dk::Viewport<2> viewport_small(tiler, Tiler::coords(1), Tiler::coords(9, 5));
	{
		int col = 0;
		const auto tilecount = bottom_layer->count();
		for (auto tile : *bottom_layer) {
			std::cout << tile;
			++col;
			if (col == tilecount.x()) {
				col = 0;
				std::cout << '\n';
			}
		}
	}

#if !defined(NDEBUG)
	std::cout << "Map size: " << tiler.map_size() << '\n';
#endif
	std::cout << "Total tiles: " << tiler.tiles_count() << '\n';
	return 0;
}
